/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu");

dojo.require('jazz.ui.Menu'); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink");

dojo.require("dm.analysis.app.ui.DiagramUtilityMethods");
var DiagramUtilityMethods = dm.analysis.app.ui.DiagramUtilityMethods;

dojo.requireLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");
var messages = dojo.i18n.getLocalization("dm.analysis.app.ui.internal.impactanalysis", "ImpactAnalysisMessages");

var EXPANSION_KEY = "dm.analysis.app.ui.internal.impactanalysis.ImpactAnalysisDiagram.expansionSequence";

/**
 * A wrapper for jazz.ui.Menu useful only for the iLOG Dojo Diagrammer nodes.
 * This class keeps enabling/disabling information about each menuItem, and will
 * apply them based on the context (i.e. the type of node that was clicked)
 */
dojo.declare("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu", [jazz.ui.Menu], 
{
	diagram: null,
	editor: null,

	_resourceLinks: null,
	_menuItems: null,
	
	constructor: function(args) {
		this.inherited(arguments);

		this.diagram = args.diagram;
		this.editor = args.editor;

		this._resourceLinks = [];
		this._initMenuItems();
	},

	_initMenuItems: function() {
		this._menuItems = new dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu_MenuItems();

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_preview(),
			disableCallback: dojo.hitch(this, this.disableCallback_preview)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_hide(),
			disableCallback: dojo.hitch(this, this.disableCallback_hide)
		});

		var subMenu = new jazz.ui.Menu();
		var menuItem = this.menuItem_hideType(subMenu);
		this._menuItems.addNodeMenuItem({
			menuItem: menuItem,
			subMenu: subMenu,
			subMenuCallback: dojo.hitch(this, this.populateSubMenu_hideType),
			disableCallback: dojo.hitch(this, this.disableCallback_hideType)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_expand(),
			disableCallback: dojo.hitch(this, this.disableCallback_expand)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_collapse(),
			disableCallback: dojo.hitch(this, this.disableCallback_collapse)
		});

		this._menuItems.addNodeMenuItem({
			menuItem: this.menuItem_debug(),
			disableCallback: dojo.hitch(this, this.disableCallback_debug)
		});

		this.startup();
	},

	menuItem_expand: function() {
		var self = this;
		return new dijit.MenuItem({
			label: messages.actionExpand,
			iconClass: 'ia-icon-expand', //$NON-NLS-1$
			disabled: false,
			onClick: function(e) {
				if (!self.diagram)
					return;

				var node = self.diagram.impactDiagram.lastMenuTarget;
				var data = node.getData();
				var uris = data.item.URI;

				if (!uris || uris.length === 0){
					console.warn("No node selected..."); //$NON-NLS-1$
					return;
				}

				var deferred = self.diagram.expandNode(uris[0], function() {
					var expansions = self.diagram.expansionSequence;
					expansions.push(uris[0]);
					self.editor._updateDirty(self.editor.editorInput.impactAnalysis.expansionSequence, expansions, EXPANSION_KEY);
				});
			}
		});

	},

	/* Target artifacts, added artifacts, and already-expanded artifacts should not be expandable */
	disableCallback_expand: function(node) {
		var data = node.getData();
		var uri  = data.item.URI[0];
		var disable = (uri == this.editor.editorInput.impactAnalysis.targetResourceUri);

		if (!disable){
			disable = dojo.some(this.editor.editorInput.impactAnalysis.addedElements, function(addedElement){
				return uri == addedElement;
			});
		}

		if (!disable){
			disable = dojo.some(this.diagram.expansionSequence, function(existingExpansionUri) {
				return (uri == existingExpansionUri);
			});
		}

		return disable;
	},

	menuItem_collapse: function() {
		var self = this;
		return new dijit.MenuItem({
			label: messages.actionCollapse,
			iconClass: 'ia-icon-collapse', //$NON-NLS-1$
			disabled: false,
			onClick: function(e) {
				if (!self.diagram)
					return;

				var node = self.diagram.impactDiagram.lastMenuTarget;
				var data = node.getData();
				var uris = data.item.URI;

				if (!uris || uris.length === 0){
					console.warn("No node selected..."); //$NON-NLS-1$
					return;
				}

				self.diagram.collapseNode(uris[0], function(collapsedUris) {
					var expansions = self.diagram.expansionSequence;
					dojo.forEach(collapsedUris, function(uri){
						expansions.splice(dojo.indexOf(expansions, uri), 1);
					});
					self.editor._updateDirty(self.editor.editorInput.impactAnalysis.expansionSequence, expansions, EXPANSION_KEY);
				});
			}
		});

	},

	/* Only nodes that were previously expanded may be collapsed */
	disableCallback_collapse: function(node) {
		var data = node.getData();
		var uri  = data.item.URI[0];
		var disable = (uri == this.editor.editorInput.impactAnalysis.targetResourceUri);

		//If it has expansionChildren (minimum requirement of links), then it is collapsible
		if (!disable){
			disable = (!data.item.expansionChildren_Links || data.item.expansionChildren_Links.length === 0);
		}

		return disable;
	},

	menuItem_debug: function() {
		var self = this;
		return new dijit.MenuItem({
			label: "Debug", //$NON-NLS-1$
			iconClass: 'ia-icon-debug', //$NON-NLS-1$
			disabled: false,
			onClick: function(e) {
				if (!self.diagram)
					return;

				var node = self.diagram.impactDiagram.lastMenuTarget;
				var data = node.getData();

				console.debug(data.item);
				console.debug(self.editor.editorInput.impactAnalysis);
			}
		});
	},

	disableCallback_debug: function(node) {
		//if debug=true is on, show this menu item
		var query = dojo.queryToObject(window.location.search.substring(1, window.location.length));
		var disableDebug = !query.debug || query.debug.toLowerCase() !== 'true'; //$NON-NLS-1$

		return disableDebug;
	},

	menuItem_preview: function() {
		var self = this;

		return new dijit.MenuItem({
			label: messages.actionPreview,
			iconClass: 'ia-icon-preview', //$NON-NLS-1$
			disabled: false,
			onClick: function(e) {
				if (!self.diagram)
					return;

				var node = self.diagram.impactDiagram.lastMenuTarget;

				self.positionPreview(node);

				var data = node.getData();
				var resourceLink = data._resourceLink;

				// Show the preview action compact rendering.
				if (resourceLink) {
					
					//override the default message when info is not available
					resourceLink.popup.disable = function() {
						if (this._isOpen) {
							this._renderError(messages.infoNotAvailable);
							this._disableOnClose = true;
							return;
						}
						if (this.onDisable)
							this.onDisable();
						this.destroy();
					};
					
					setTimeout(dojo.hitch(resourceLink, function() {
						if (this.popup && !this.popup._isOpen) {
							this.popup.open();
						}
					}), 10);
				}
			}
		});
	},

	disableCallback_preview: function(node) {
		var data = node.getData();
		var disable = !data._resourceLink;
		return disable;
	},

	menuItem_hide: function() {
		var self = this;

		return new dijit.MenuItem({
			label: messages.actionHideArtifact,
			iconClass: 'ia-icon-hide-artifact', //$NON-NLS-1$
			disabled: false,
			onClick: function() {
				var node = self.diagram.impactDiagram.lastMenuTarget;
				var data = node.getData();

				if (data.item.URI.length != 1){
					console.warn("For removal, exactly one element should be selected. " + data.item.URI.length + " selected instead.");
					return;
				}

				var uri  = data.item.URI[0];
				var name = data.item.name[0];
				var type = (data.item.type && data.item.type[0]) ? data.item.type[0] : "";
				
				// Determine the type of the resource
				type = self._determineTypeName(type);

				// Confirm with the user before removing the element					
				if (confirm(messages.confirmHideArtifact)) {
					self.diagram.removeElement(uri, name, type);
				}
			}
		});
	},

	disableCallback_hide: function(node) {
		var data = node.getData();
		var uri  = data.item.URI;
		var disable = (uri == this.editor.editorInput.impactAnalysis.targetResourceUri);

		if (!disable){
			disable = dojo.some(this.editor.editorInput.impactAnalysis.addedElements, function(addedElement){
				return uri == addedElement;
			});
		}

		return disable;
	},

	menuItem_hideType: function(subMenu) {
		var self = this;

		return new dijit.PopupMenuItem({
			label: messages.actionHideType,
			iconClass: 'ia-icon-hide-type',
			popup: subMenu
		});
	},

	populateSubMenu_hideType: function(subMenu) {
		if (!subMenu){
			console.warn("Unable to populate sub-menu. It appears to be missing."); //$NON-NLS-1$
			return;
		}

		var node = this.diagram.impactDiagram.lastMenuTarget;
		var data = node.getData();

		if (data.item.URI.length != 1){
			console.warn("For quick filtering, exactly one element should be selected. " + data.item.URI.length + " selected instead."); //$NON-NLS-1$  //$NON-NLS-2$
			return;
		}

		var uri  = data.item.URI[0];
		var name = data.item.name[0];
		var type = (data.item.type && data.item.type[0]) ? data.item.type[0] : ""; //this is from the node, set by the server, but the server only passes it in some DM-specific cases
		var rdfTypes = (data.item.rdfTypes) ? data.item.rdfTypes : [type];

		if (subMenu.hasChildren()){
			dojo.forEach(subMenu.getChildren(), function (child) {
				subMenu.removeChild(child);
			});
		}

		var self = this;
		dojo.forEach(rdfTypes, function (rdfType) {
			var typeName = self._determineTypeName(rdfType);

			subMenu.addChild(new dijit.MenuItem({
				label: typeName,
				//iconClass: 'ia-icon-filter',
				disabled: false,
				onClick: function() {
					if(confirm(dojo.string.substitute(messages.confirmHideType, [typeName]))) {
						self.diagram.addArtifactQuickFilter(uri, name, rdfType, typeName);
					}
				}
			}));
		});
	},

	disableCallback_hideType: function(node) {
		var data = node.getData();
		var uri  = data.item.URI;
		var disable = (uri == this.editor.editorInput.impactAnalysis.targetResourceUri);

		if (!disable){
			disable = dojo.some(this.editor.editorInput.impactAnalysis.addedElements, function(addedElement){
				return uri == addedElement;
			});
		}

		if (!disable){
			var hasDmType = data.item.type && !!data.item.type[0];
			var hasRdfType = data.item.rdfTypes && !!data.item.rdfTypes[0];
			disable = !hasDmType && !hasRdfType;
		}

		return disable;
	},

	_determineTypeName: function(uri){
		var split;
		var typeName = messages.unknownName;
		if (uri.indexOf("/") > 0) {
			split = uri.split("/");
			typeName = split[split.length-1];
		}
		if (uri.indexOf("#") > 0) {
			split = uri.split("#");
			typeName = split[split.length-1];
		}

		return typeName;
	},


	positionPreview: function(node){
		var resourceLinkDiv = this.createPreviewResourceLink(node);
		
		// Determine the coordinates for the popup menu.
		// The bounds for the node are relative to its ilog diagram.
		// Adjust the coordinates for the actual screen
		var viewport = this.diagram.impactDiagram.getViewport();
		var bounds = node.getBounds(null);
		var nodePos = viewport.pointContentToClient(bounds);
		var diagramPos = dojo.position(this.diagram.impactDiagram.domNode);
		var topLeft_x = Math.round(nodePos.x + diagramPos.x);
		var topLeft_y = Math.round(nodePos.y + diagramPos.y);
		
		// Set the position of the preview 'div" node to the coordinates 
		// of the top left corner of the graph element.
		if (resourceLinkDiv) {
			dojo.style(resourceLinkDiv, { "left": topLeft_x+'px', "top": topLeft_y+'px' });
		}
	},


	/** Create the preview resource link widget. The resource
	 *  link widget will position itself near to its parent
	 *  node. To accomplish this, a special div node is 
	 *  created as a child of the graph element's raw node
	 *  but it is will be given an absolute position on the
	 *  page just before the pop-up menu is displayed.
	 *
	 * @param graphElement the analysis Node or Link.
	 * @return the 'div" node for positioning the resource link widget (or null)
	 */
	createPreviewResourceLink: function(graphElement) {
		var data = graphElement.getData();
		
		// If the resource link has not been created yet
		if (!data._resourceLink) {
			// Compute the URI for rendering the resource information
			var resourceUri = data.item.URI[0];
			var renderingUri = DiagramUtilityMethods.call_getCompactRenderingUri(resourceUri);
		
			// Compute the presentation URI to open the resource
			var presentationUri = DiagramUtilityMethods.call_getResourceEditUri(resourceUri);
			if (!presentationUri) {
				presentationUri = resourceUri;
			}
			
			// If the resource does not have a URI for compact rendering
			if (!renderingUri) {
				// Disable the preview action for this resource
				data._resourceLink = null;
				data._resourceLinkDiv = null;
			}
			
			else {		
				// Create a <div> node that will be the parent of the 
				// resource link widget. This div node and the resource
				// link are created once per graph element but the div
				// node will be positioned each time the pop-up menu
				// is displayed to absolute coordinates.
				var domNode = this.diagram.domNode;
				var div = document.createElement("div");
				dojo.style(div, {
					"visibility": "hidden",
					"display":    "block",
					"position":   "fixed",
					"width":      "5px",
					"height":     "5px"
				});
				div.className = "resourceLinkDiv";
				domNode.appendChild(div);
				
				// Use our own 'srcNodeRef' so that we can set a target
				// which opens the link in a new window/tab
				var link = dojo.create('a', {
					href: presentationUri,
					target: '_blank'
				});

				// Create the resource link
				var resourceLink = new jazz.ui.ResourceLink({
					uri: renderingUri,
					presentationUri: presentationUri,
					lazyFetch : true,
					label: "-",
				}, link);
				
				div.appendChild(resourceLink.domNode);
				resourceLink.startup();
		
				// Save the resource link widget
				data._resourceLink = resourceLink;
				data._resourceLinkDiv = div;

				this._resourceLinks.push(resourceLink);
			}
		}

		// Return the 'div' node to be positioned
		return data._resourceLinkDiv;
	},

	/** 
	 * Call this to reset the items in the visible menu based on the node that was clicked
	 */
	updateMenuForNode: function(node) {
		//If it hasn't already been done, attempt to create a div for the Preview and place it at the node
		//It won't be shown yet, we just need to get it ready.
		this.positionPreview(node);

		var self = this;

		//Remove everything from the visible menu
		dojo.forEach(this.getChildren(), function(child){
			self.removeChild(child);
		});

		//Add everything from our stored items into the visible menu, check with disableCallback
		//first to make sure you should, then populate the sub-menus for any menu item that has one
		dojo.forEach(this._menuItems.getNodeMenuItems(), function(nodeMenuItem) { 
			if (!nodeMenuItem.disableCallback(node)){
				self.addChild(nodeMenuItem.menuItem);
			}
			if (nodeMenuItem.subMenuCallback){
				nodeMenuItem.subMenuCallback(nodeMenuItem.subMenu);
			}
		});

	},

	/**
	 * Open the Menu at the specified node. 
	 * Useful for anytime you want to open the menu without a click.
	 */
	openAtNode: function(node) {
		this.diagram.impactDiagram.lastMenuTarget = node;
		this.diagram.impactDiagram.nodeMenu.updateMenuForNode(node);

		this._openMyself({
			target: node.rawNode
		});
	},

	/** 
	 * When the diagram nodes and links are destroyed, the floating resource links need to be deleted.
	 */
	cleanup: function() {
		// Destroy all of the resource links
		dojo.forEach(this._resourceLinks, function(resourceLink, i) { resourceLink.destroy(); });
		this._resourceLinks = [];
		
		// Remove all of the special 'div' resource link nodes from the diagram.
		var diagramDomNode = this.diagram.domNode;
		dojo.query(('.resourceLinkDiv'), diagramDomNode).forEach(function(divNode, index) {
			diagramDomNode.removeChild(divNode);
		});
	},
	
	/** 
	 * DiagramNodeMenu.destroyRecursive() shouldn't ever be called, since the menuItems aren't always
	 * ALL in the Menu, but they are always all in this._menuItems, so let us handle destroying them.
	 */
	destroy: function() {
		this.cleanup();
		this._menuItems.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("dm.analysis.app.ui.internal.impactanalysis.DiagramNodeMenu_MenuItems", null, {

	_nodeMenuItems: [],

	constructor: function() {
		this._nodeMenuItems = [];
	},
	
	addNodeMenuItem: function(args) {
		if (args.menuItem && dojo.isFunction(args.disableCallback)) {
			var nodeMenuItem = {
				menuItem: args.menuItem,
				subMenu: args.subMenu,
				subMenuCallback: args.subMenuCallback,
				disableCallback: args.disableCallback
			};

			this._nodeMenuItems.push(nodeMenuItem);
		}
	},

	getNodeMenuItems: function() {
		return this._nodeMenuItems;
	},

	destroy: function() {
		dojo.forEach(this._nodeMenuItems, function(nodeMenuItem) { 
			nodeMenuItem.menuItem.destroyRecursive();
		});
	}
	  
});

})();
